@@ -1,11 +1,11 @@
 from __future__ import annotations
 
+import contextlib
 from functools import reduce
 from operator import and_
 from typing import TYPE_CHECKING
 from typing import Any
 from typing import Iterator
-from typing import Literal
 from typing import Mapping
 from typing import Sequence
 
@@ -21,11 +21,10 @@ from narwhals._duckdb.utils import native_to_narwhals_dtype
 from narwhals.dependencies import get_duckdb
 from narwhals.exceptions import ColumnNotFoundError
 from narwhals.exceptions import InvalidOperationError
-from narwhals.typing import CompliantDataFrame
+from narwhals.typing import CompliantLazyFrame
 from narwhals.utils import Implementation
 from narwhals.utils import Version
 from narwhals.utils import generate_temporary_column_name
-from narwhals.utils import import_dtypes_module
 from narwhals.utils import not_implemented
 from narwhals.utils import parse_columns_to_drop
 from narwhals.utils import parse_version
@@ -37,21 +36,36 @@ if TYPE_CHECKING:
     import pandas as pd
     import pyarrow as pa
     from typing_extensions import Self
+    from typing_extensions import TypeIs
 
+    from narwhals._compliant.typing import CompliantDataFrameAny
     from narwhals._duckdb.expr import DuckDBExpr
     from narwhals._duckdb.group_by import DuckDBGroupBy
     from narwhals._duckdb.namespace import DuckDBNamespace
     from narwhals._duckdb.series import DuckDBInterchangeSeries
+    from narwhals.dataframe import LazyFrame
     from narwhals.dtypes import DType
+    from narwhals.stable.v1 import DataFrame as DataFrameV1
+    from narwhals.typing import AsofJoinStrategy
+    from narwhals.typing import JoinStrategy
+    from narwhals.typing import LazyUniqueKeepStrategy
+    from narwhals.utils import _FullContext
 
-from narwhals.typing import CompliantLazyFrame
+with contextlib.suppress(ImportError):  # requires duckdb>=1.3.0
+    from duckdb import SQLExpression  # type: ignore[attr-defined, unused-ignore]
 
 
-class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"]):
+class DuckDBLazyFrame(
+    CompliantLazyFrame[
+        "DuckDBExpr",
+        "duckdb.DuckDBPyRelation",
+        "LazyFrame[duckdb.DuckDBPyRelation] | DataFrameV1[duckdb.DuckDBPyRelation]",
+    ]
+):
     _implementation = Implementation.DUCKDB
 
     def __init__(
-        self: Self,
+        self,
         df: duckdb.DuckDBPyRelation,
         *,
         backend_version: tuple[int, ...],
@@ -61,42 +75,63 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
         self._version = version
         self._backend_version = backend_version
         self._cached_schema: dict[str, DType] | None = None
+        self._cached_columns: list[str] | None = None
         validate_backend_version(self._implementation, self._backend_version)
 
-    def __narwhals_dataframe__(self: Self) -> Self:  # pragma: no cover
+    @staticmethod
+    def _is_native(obj: duckdb.DuckDBPyRelation | Any) -> TypeIs[duckdb.DuckDBPyRelation]:
+        return isinstance(obj, duckdb.DuckDBPyRelation)
+
+    @classmethod
+    def from_native(
+        cls, data: duckdb.DuckDBPyRelation, /, *, context: _FullContext
+    ) -> Self:
+        return cls(
+            data, backend_version=context._backend_version, version=context._version
+        )
+
+    def to_narwhals(
+        self, *args: Any, **kwds: Any
+    ) -> LazyFrame[duckdb.DuckDBPyRelation] | DataFrameV1[duckdb.DuckDBPyRelation]:
+        if self._version is Version.MAIN:
+            return self._version.lazyframe(self, level="lazy")
+
+        from narwhals.stable.v1 import DataFrame as DataFrameV1
+
+        return DataFrameV1(self, level="interchange")  # type: ignore[no-any-return]
+
+    def __narwhals_dataframe__(self) -> Self:  # pragma: no cover
         # Keep around for backcompat.
         if self._version is not Version.V1:
             msg = "__narwhals_dataframe__ is not implemented for DuckDBLazyFrame"
             raise AttributeError(msg)
         return self
 
-    def __narwhals_lazyframe__(self: Self) -> Self:
+    def __narwhals_lazyframe__(self) -> Self:
         return self
 
-    def __native_namespace__(self: Self) -> ModuleType:
+    def __native_namespace__(self) -> ModuleType:
         return get_duckdb()  # type: ignore[no-any-return]
 
-    def __narwhals_namespace__(self: Self) -> DuckDBNamespace:
+    def __narwhals_namespace__(self) -> DuckDBNamespace:
         from narwhals._duckdb.namespace import DuckDBNamespace
 
         return DuckDBNamespace(
             backend_version=self._backend_version, version=self._version
         )
 
-    def __getitem__(self: Self, item: str) -> DuckDBInterchangeSeries:
+    def get_column(self, name: str) -> DuckDBInterchangeSeries:
         from narwhals._duckdb.series import DuckDBInterchangeSeries
 
-        return DuckDBInterchangeSeries(self.native.select(item), version=self._version)
+        return DuckDBInterchangeSeries(self.native.select(name), version=self._version)
 
     def _iter_columns(self) -> Iterator[duckdb.Expression]:
         for name in self.columns:
             yield col(name)
 
     def collect(
-        self: Self,
-        backend: ModuleType | Implementation | str | None,
-        **kwargs: Any,
-    ) -> CompliantDataFrame[Any, Any, Any]:
+        self, backend: ModuleType | Implementation | str | None, **kwargs: Any
+    ) -> CompliantDataFrameAny:
         if backend is None or backend is Implementation.PYARROW:
             import pyarrow as pa  # ignore-banned-import
 
@@ -134,29 +169,29 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
         msg = f"Unsupported `backend` value: {backend}"  # pragma: no cover
         raise ValueError(msg)  # pragma: no cover
 
-    def head(self: Self, n: int) -> Self:
+    def head(self, n: int) -> Self:
         return self._with_native(self.native.limit(n))
 
     def simple_select(self, *column_names: str) -> Self:
         return self._with_native(self.native.select(*column_names))
 
-    def aggregate(self: Self, *exprs: DuckDBExpr) -> Self:
+    def aggregate(self, *exprs: DuckDBExpr) -> Self:
         selection = [val.alias(name) for name, val in evaluate_exprs(self, *exprs)]
         return self._with_native(self.native.aggregate(selection))  # type: ignore[arg-type]
 
     def select(
-        self: Self,
+        self,
         *exprs: DuckDBExpr,
     ) -> Self:
         selection = (val.alias(name) for name, val in evaluate_exprs(self, *exprs))
         return self._with_native(self.native.select(*selection))
 
-    def drop(self: Self, columns: Sequence[str], *, strict: bool) -> Self:
+    def drop(self, columns: Sequence[str], *, strict: bool) -> Self:
         columns_to_drop = parse_columns_to_drop(self, columns=columns, strict=strict)
         selection = (name for name in self.columns if name not in columns_to_drop)
         return self._with_native(self.native.select(*selection))
 
-    def lazy(self: Self, *, backend: Implementation | None = None) -> Self:
+    def lazy(self, *, backend: Implementation | None = None) -> Self:
         # The `backend`` argument has no effect but we keep it here for
         # backwards compatibility because in `narwhals.stable.v1`
         # function `.from_native()` will return a DataFrame for DuckDB.
@@ -166,29 +201,29 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
             raise ValueError(msg)
         return self
 
-    def with_columns(self: Self, *exprs: DuckDBExpr) -> Self:
+    def with_columns(self, *exprs: DuckDBExpr) -> Self:
         new_columns_map = dict(evaluate_exprs(self, *exprs))
         result = [
             new_columns_map.pop(name).alias(name)
             if name in new_columns_map
             else col(name)
-            for name in self.native.columns
+            for name in self.columns
         ]
         result.extend(value.alias(name) for name, value in new_columns_map.items())
         return self._with_native(self.native.select(*result))
 
-    def filter(self: Self, predicate: DuckDBExpr) -> Self:
+    def filter(self, predicate: DuckDBExpr) -> Self:
         # `[0]` is safe as the predicate's expression only returns a single column
         mask = predicate(self)[0]
         return self._with_native(self.native.filter(mask))
 
     @property
-    def schema(self: Self) -> dict[str, DType]:
+    def schema(self) -> dict[str, DType]:
         if self._cached_schema is None:
             # Note: prefer `self._cached_schema` over `functools.cached_property`
             # due to Python3.13 failures.
             self._cached_schema = {
-                column_name: native_to_narwhals_dtype(str(duckdb_dtype), self._version)
+                column_name: native_to_narwhals_dtype(duckdb_dtype, self._version)
                 for column_name, duckdb_dtype in zip(
                     self.native.columns, self.native.types
                 )
@@ -196,51 +231,59 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
         return self._cached_schema
 
     @property
-    def columns(self: Self) -> list[str]:
-        return list(self.schema)
+    def columns(self) -> list[str]:
+        if self._cached_columns is None:
+            self._cached_columns = (
+                list(self.schema)
+                if self._cached_schema is not None
+                else self.native.columns
+            )
+        return self._cached_columns
 
-    def to_pandas(self: Self) -> pd.DataFrame:
+    def to_pandas(self) -> pd.DataFrame:
         # only if version is v1, keep around for backcompat
         import pandas as pd  # ignore-banned-import()
 
         if parse_version(pd) >= (1, 0, 0):
             return self.native.df()
         else:  # pragma: no cover
-            msg = f"Conversion to pandas requires pandas>=1.0.0, found {pd.__version__}"
+            msg = f"Conversion to pandas requires 'pandas>=1.0.0', found {pd.__version__}"
             raise NotImplementedError(msg)
 
-    def to_arrow(self: Self) -> pa.Table:
+    def to_arrow(self) -> pa.Table:
         # only if version is v1, keep around for backcompat
         return self.native.arrow()
 
-    def _with_version(self: Self, version: Version) -> Self:
+    def _with_version(self, version: Version) -> Self:
         return self.__class__(
             self.native, version=version, backend_version=self._backend_version
         )
 
-    def _with_native(self: Self, df: duckdb.DuckDBPyRelation) -> Self:
+    def _with_native(self, df: duckdb.DuckDBPyRelation) -> Self:
         return self.__class__(
             df, backend_version=self._backend_version, version=self._version
         )
 
-    def group_by(self: Self, *keys: str, drop_null_keys: bool) -> DuckDBGroupBy:
+    def group_by(
+        self, keys: Sequence[str] | Sequence[DuckDBExpr], *, drop_null_keys: bool
+    ) -> DuckDBGroupBy:
         from narwhals._duckdb.group_by import DuckDBGroupBy
 
         return DuckDBGroupBy(self, keys, drop_null_keys=drop_null_keys)
 
-    def rename(self: Self, mapping: Mapping[str, str]) -> Self:
+    def rename(self, mapping: Mapping[str, str]) -> Self:
         df = self.native
-        selection = [
-            f"{name} as {mapping[name]}" if name in mapping else name
+        selection = (
+            col(name).alias(mapping[name]) if name in mapping else col(name)
             for name in df.columns
-        ]
-        return self._with_native(df.select(", ".join(selection)))
+        )
+        return self._with_native(self.native.select(*selection))
 
     def join(
-        self: Self,
+        self,
         other: Self,
         *,
-        how: Literal["inner", "left", "full", "cross", "semi", "anti"],
+        how: JoinStrategy,
         left_on: Sequence[str] | None,
         right_on: Sequence[str] | None,
         suffix: str,
@@ -249,26 +292,29 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
 
         if native_how == "cross":
             if self._backend_version < (1, 1, 4):
-                msg = f"DuckDB>=1.1.4 is required for cross-join, found version: {self._backend_version}"
+                msg = f"'duckdb>=1.1.4' is required for cross-join, found version: {self._backend_version}"
                 raise NotImplementedError(msg)
-            rel = self.native.set_alias("lhs").cross(
-                other.native.set_alias("rhs")
-            )  # pragma: no cover
+            rel = self.native.set_alias("lhs").cross(other.native.set_alias("rhs"))
         else:
             # help mypy
             assert left_on is not None  # noqa: S101
             assert right_on is not None  # noqa: S101
-            condition = " and ".join(
-                f'lhs."{left}" = rhs."{right}"' for left, right in zip(left_on, right_on)
+            it = (
+                col(f'lhs."{left}"') == col(f'rhs."{right}"')
+                for left, right in zip(left_on, right_on)
             )
+            condition: duckdb.Expression = reduce(and_, it)
             rel = self.native.set_alias("lhs").join(
-                other.native.set_alias("rhs"), condition=condition, how=native_how
+                other.native.set_alias("rhs"),
+                # NOTE: Fixed in `--pre` https://github.com/duckdb/duckdb/pull/16933
+                condition=condition,  # type: ignore[arg-type, unused-ignore]
+                how=native_how,
             )
 
         if native_how in {"inner", "left", "cross", "outer"}:
-            select = [col(f'lhs."{x}"') for x in self.native.columns]
-            for name in other.native.columns:
-                col_in_lhs: bool = name in self.native.columns
+            select = [col(f'lhs."{x}"') for x in self.columns]
+            for name in other.columns:
+                col_in_lhs: bool = name in self.columns
                 if native_how == "outer" and not col_in_lhs:
                     select.append(col(f'rhs."{name}"'))
                 elif (native_how == "outer") or (
@@ -284,33 +330,34 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
         return self._with_native(res)
 
     def join_asof(
-        self: Self,
+        self,
         other: Self,
         *,
         left_on: str | None,
         right_on: str | None,
         by_left: Sequence[str] | None,
         by_right: Sequence[str] | None,
-        strategy: Literal["backward", "forward", "nearest"],
+        strategy: AsofJoinStrategy,
         suffix: str,
     ) -> Self:
         lhs = self.native
         rhs = other.native
-        conditions = []
+        conditions: list[duckdb.Expression] = []
         if by_left is not None and by_right is not None:
-            conditions += [
-                f'lhs."{left}" = rhs."{right}"' for left, right in zip(by_left, by_right)
-            ]
+            conditions.extend(
+                col(f'lhs."{left}"') == col(f'rhs."{right}"')
+                for left, right in zip(by_left, by_right)
+            )
         else:
             by_left = by_right = []
         if strategy == "backward":
-            conditions += [f'lhs."{left_on}" >= rhs."{right_on}"']
+            conditions.append(col(f'lhs."{left_on}"') >= col(f'rhs."{right_on}"'))
         elif strategy == "forward":
-            conditions += [f'lhs."{left_on}" <= rhs."{right_on}"']
+            conditions.append(col(f'lhs."{left_on}"') <= col(f'rhs."{right_on}"'))
         else:
             msg = "Only 'backward' and 'forward' strategies are currently supported for DuckDB"
             raise NotImplementedError(msg)
-        condition = " and ".join(conditions)
+        condition: duckdb.Expression = reduce(and_, conditions)
         select = ["lhs.*"]
         for name in rhs.columns:
             if name in lhs.columns and (
@@ -318,7 +365,9 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
             ):
                 select.append(f'rhs."{name}" as "{name}{suffix}"')
             elif right_on is None or name not in {right_on, *by_right}:
-                select.append(f'"{name}"')
+                select.append(str(col(name)))
+        # Replace with Python API call once
+        # https://github.com/duckdb/duckdb/discussions/16947 is addressed.
         query = f"""
             SELECT {",".join(select)}
             FROM lhs
@@ -327,44 +376,41 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
             """  # noqa: S608
         return self._with_native(duckdb.sql(query))
 
-    def collect_schema(self: Self) -> dict[str, DType]:
-        return {
-            column_name: native_to_narwhals_dtype(str(duckdb_dtype), self._version)
-            for column_name, duckdb_dtype in zip(self.native.columns, self.native.types)
-        }
+    def collect_schema(self) -> dict[str, DType]:
+        return self.schema
 
     def unique(
-        self: Self, subset: Sequence[str] | None, *, keep: Literal["any", "none"]
+        self, subset: Sequence[str] | None, *, keep: LazyUniqueKeepStrategy
     ) -> Self:
-        if subset is not None:
-            rel = self.native
+        if subset_ := subset if keep == "any" else (subset or self.columns):
+            if self._backend_version < (1, 3):
+                msg = (
+                    "At least version 1.3 of DuckDB is required for `unique` operation\n"
+                    "with `subset` specified."
+                )
+                raise NotImplementedError(msg)
             # Sanitise input
-            if any(x not in rel.columns for x in subset):
-                msg = f"Columns {set(subset).difference(rel.columns)} not found in {rel.columns}."
+            if any(x not in self.columns for x in subset_):
+                msg = f"Columns {set(subset_).difference(self.columns)} not found in {self.columns}."
                 raise ColumnNotFoundError(msg)
-            idx_name = generate_temporary_column_name(8, rel.columns)
-            count_name = generate_temporary_column_name(8, [*rel.columns, idx_name])
-            if keep == "none":
-                keep_condition = col(count_name) == lit(1)
-            else:
-                keep_condition = col(idx_name) == lit(1)
-            partition_by_sql = generate_partition_by_sql(*subset)
-            query = f"""
-                select *,
-                        row_number() over ({partition_by_sql}) as "{idx_name}",
-                        count(*) over ({partition_by_sql}) as "{count_name}"
-                from rel
-                """  # noqa: S608
+            idx_name = generate_temporary_column_name(8, self.columns)
+            count_name = generate_temporary_column_name(8, [*self.columns, idx_name])
+            partition_by_sql = generate_partition_by_sql(*(subset_))
+            name = count_name if keep == "none" else idx_name
+            idx_expr = SQLExpression(
+                f"{FunctionExpression('row_number')} over ({partition_by_sql})"
+            ).alias(idx_name)
+            count_expr = SQLExpression(
+                f"{FunctionExpression('count', StarExpression())} over ({partition_by_sql})"
+            ).alias(count_name)
             return self._with_native(
-                duckdb.sql(query)
-                .filter(keep_condition)
+                self.native.select(StarExpression(), idx_expr, count_expr)
+                .filter(col(name) == lit(1))
                 .select(StarExpression(exclude=[count_name, idx_name]))
             )
         return self._with_native(self.native.unique(", ".join(self.columns)))
 
-    def sort(
-        self: Self, *by: str, descending: bool | Sequence[bool], nulls_last: bool
-    ) -> Self:
+    def sort(self, *by: str, descending: bool | Sequence[bool], nulls_last: bool) -> Self:
         if isinstance(descending, bool):
             descending = [descending] * len(by)
         if nulls_last:
@@ -379,14 +425,13 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
             )
         return self._with_native(self.native.sort(*it))
 
-    def drop_nulls(self: Self, subset: Sequence[str] | None) -> Self:
-        rel = self.native
-        subset_ = subset if subset is not None else rel.columns
+    def drop_nulls(self, subset: Sequence[str] | None) -> Self:
+        subset_ = subset if subset is not None else self.columns
         keep_condition = reduce(and_, (col(name).isnotnull() for name in subset_))
         return self._with_native(self.native.filter(keep_condition))
 
-    def explode(self: Self, columns: Sequence[str]) -> Self:
-        dtypes = import_dtypes_module(self._version)
+    def explode(self, columns: Sequence[str]) -> Self:
+        dtypes = self._version.dtypes
         schema = self.collect_schema()
         for name in columns:
             dtype = schema[name]
@@ -430,7 +475,7 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
         return self._with_native(non_null_rel.union(null_rel))
 
     def unpivot(
-        self: Self,
+        self,
         on: Sequence[str] | None,
         index: Sequence[str] | None,
         variable_name: str,
@@ -447,8 +492,10 @@ class DuckDBLazyFrame(CompliantLazyFrame["DuckDBExpr", "duckdb.DuckDBPyRelation"
             msg = "`value_name` cannot be empty string for duckdb backend."
             raise NotImplementedError(msg)
 
-        unpivot_on = ", ".join(f'"{name}"' for name in on_)
+        unpivot_on = ", ".join(str(col(name)) for name in on_)
         rel = self.native  # noqa: F841
+        # Replace with Python API once
+        # https://github.com/duckdb/duckdb/discussions/16980 is addressed.
         query = f"""
             unpivot rel
             on {unpivot_on}
